#!/bin/sh

echo "sMockin requires Java 11 or later to run"

APP_NAME="sMockin!"
APP_VERSION="2.19.2"

APP_DIR_PATH="${HOME}/.smockin"
DB_DIR_PATH="${APP_DIR_PATH}/db"
PIDS_DIR_PATH="${APP_DIR_PATH}/pids"
DB_DRIVER_DIR_PATH="${DB_DIR_PATH}/driver"
DB_DATA_DIR_PATH="${DB_DIR_PATH}/data"
DB_MARKER="db_initialized"
H2_JAR_NAME="h2-2.1.212.jar"
DB_PROPS_FILE=db.properties
APP_PROPS_FILE=app.properties
SMOCKIN_PID_FILE="$PIDS_DIR_PATH/smockin-app.pid"
H2_DB_PID_FILE="$PIDS_DIR_PATH/smockin-db.pid"

RESET_SYS_ADMIN=false
MULTI_USER_MODE=false

if [ ! -d "${APP_DIR_PATH}" ]
then
  ./install.sh
  sleep 3
fi



#
# Check if smockin already has a pid.
if [ -f $SMOCKIN_PID_FILE ];
then
    echo "SMOCKIN appears to already running, restarting this service..."
    ./shutdown.sh
    sleep 5
fi

# DB properties
DB_PROPS_FILE=$(grep "^[^#;]" ${DB_DIR_PATH}/${DB_PROPS_FILE})

DB_USERNAME=$(echo "$DB_PROPS_FILE" | grep "DB_USERNAME" | awk '{ print $3 }')
DB_PASSWORD=$(echo "$DB_PROPS_FILE" | grep "DB_PASSWORD" | awk '{ print $3 }')
DRIVER_CLASS=$(echo "$DB_PROPS_FILE" | grep "DRIVER_CLASS" | awk '{ print $3 }')
JDBC_URL=$(echo "$DB_PROPS_FILE" | grep "JDBC_URL" | awk '{ print $3 }')
HIBERNATE_DIALECT=$(echo "$DB_PROPS_FILE" | grep "HIBERNATE_DIALECT" | awk '{ print $3 }')
MAX_POOL_SIZE=$(echo "$DB_PROPS_FILE" | grep "MAX_POOL_SIZE" | awk '{ print $3 }')
MIN_POOL_SIZE=$(echo "$DB_PROPS_FILE" | grep "MIN_POOL_SIZE" | awk '{ print $3 }')

# APP properties
APP_PROPS_FILE=$(grep "^[^#;]" ${APP_DIR_PATH}/${APP_PROPS_FILE})

H2_PORT=$(echo "$APP_PROPS_FILE" | grep "H2_PORT" | awk '{ print $3 }')
APP_PORT=$(echo "$APP_PROPS_FILE" | grep "APP_PORT" | awk '{ print $3 }')
MULTI_USER_MODE_CONF=$(echo "$APP_PROPS_FILE" | grep "MULTI_USER_MODE" | awk '{ print $3 }')

if ([ ! -z $MULTI_USER_MODE_CONF ] && [ $MULTI_USER_MODE_CONF = "TRUE" ])
then
    MULTI_USER_MODE=true
fi

if ([ ! -z "$1" ] && [ $1 = "-RESET_SYS_ADMIN" ]) || ([ ! -z "$2" ] && [ $2 = "-RESET_SYS_ADMIN" ]); then
    RESET_SYS_ADMIN=true
fi

echo "#####################################################################################"
echo "# "
echo "#  $APP_NAME"
echo "# "
echo "#  v$APP_VERSION"
echo "#  "


#
# Check for H2 DB Server driver and start it up (in TCP server mode) if not already running
#
if ([ $DRIVER_CLASS = "org.h2.Driver" ] && [ !$USE_INMEM_DB ]);
then

  H2_PID=$(ps aux | grep SmockinH2DB | grep -v grep | awk '{print $2}')
  JDBC_URL=$(echo $JDBC_URL | sed "s/{H2.PORT}/$H2_PORT/g")

  if [ ! -z "$H2_PID" ] 
  then
    echo "#  H2 TCP Database is already running..."
  else
    echo "#  Starting H2 TCP Database..."
    java -cp $DB_DRIVER_DIR_PATH/$H2_JAR_NAME -DSmockinH2DB org.h2.tools.Server -tcp -tcpAllowOthers -tcpPort $H2_PORT > /dev/null 2>&1 &
    echo "$!" > $H2_DB_PID_FILE
  fi

fi


echo "#  JDBC Connectivity Properties:"
echo "#  - JDBC DRIVER: $DRIVER_CLASS"
echo "#  - JDBC URL: $JDBC_URL"
echo "#"


#
# Prepare runtime args
#
VM_ARGS="-Dspring.datasource.url=$JDBC_URL -Dspring.datasource.username=$DB_USERNAME -Dspring.datasource.password=$DB_PASSWORD -Dspring.datasource.maximumPoolSize=$MAX_POOL_SIZE -Dspring.datasource.minimumIdle=$MIN_POOL_SIZE -Duser.timezone=UTC -Dapp.version=$APP_VERSION"
APP_PROFILE="production"
RESET_SYS_ADMIN_ARG=""

if ( $RESET_SYS_ADMIN ); then
  RESET_SYS_ADMIN_ARG="-Dreset.sys.admin=true"
fi


#
# Override any proxy environment variables, as this can cause 403 issues with internal S3 client and S3 mock server
#
export HTTP_PROXY=
export HTTPS_PROXY=


#
# START UP SMOCKIN APPLICATION
# (JPA WILL CREATE THE ACTUAL SMOCKIN DB AUTOMATICALLY IF IT DOES NOT ALREADY EXIST)
#
echo "#"
echo "#  Starting Main Application..."
echo "#"

if ( $MULTI_USER_MODE ); then
  echo "#  The application is running in 'Multi User Mode'"
  echo "#"
fi

echo "#  Please Note:"
echo "#  - Application logs are available from: .smockin/log (under the user.home directory)"
echo "#  - Navigate to: 'http://localhost:$APP_PORT/index.html' to access the Smockin Admin UI."


echo "#  - Run 'shutdown.sh' when you wish to terminate this application."
java -Dspring.profiles.active=$APP_PROFILE -Dserver.port=$APP_PORT -Dmulti.user.mode=$MULTI_USER_MODE $VM_ARGS $RESET_SYS_ADMIN_ARG -jar lib/smockin-$APP_VERSION.jar > /dev/null 2>&1 &
echo "$!" > $SMOCKIN_PID_FILE


echo "#"
echo "#####################################################################################"
